// SPDX-FileCopyrightText: 2020 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

form.object-features {
  span.description {
    text-align: left;
  }

  .control-label-group {
    text-align: left;
    margin-right: 0;
    width: @name-value-table-name-width;
    color: @text-color-light;

    label {
      font-size: inherit;
    }
  }

  .control-group {
    margin-top: 0;
    margin-bottom: 0;

    &.indeterminate {
      justify-content: flex-start;

      .control-label-group {
        flex: 0 1 auto;
      }

      select {
        width: auto;
        flex: 0 1 auto;

        & + span.hint {
          flex: 0 1 auto;
        }
      }
    }
  }

  .toggle-switch {
    margin-left: @table-column-padding;
  }

  select {
    margin-right: .5em;
    margin-left: @table-column-padding;

    & + span.hint {
      margin: .35em;
      color: @gray-light;
      font-style: italic;
    }
  }
}
