<?php

// SPDX-FileCopyrightText: 2021 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

namespace Icinga\Module\Icingadb\Command\Object;

/**
 * Schedule a host downtime
 *
 * @deprecated Use {@see ScheduleDowntimeCommand} instead
 */
class ScheduleHostDowntimeCommand extends ScheduleServiceDowntimeCommand
{
}
