// SPDX-FileCopyrightText: 2020 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

.downtime-progress {
  .progress-bar();

  &.progress-bar .timeline .downtime-elapsed {
    &::before,
    > .bar {
      background-color: @state-ok;
    }
  }
}
