// SPDX-FileCopyrightText: 2020 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

.object-detail-actions a {
  border-bottom: 1px solid @gray-light;
  display: inline-block;
  margin-bottom: .25em;

  .text-ellipsis();
  max-width: 32em;

  &:hover {
    border-color: @icinga-blue-light;
    color: @icinga-blue;
    text-decoration: none;
  }
}

ul.object-detail-actions {
    list-style-type: none;
    padding: 0;
    margin: 0;
}
