// SPDX-FileCopyrightText: 2025 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

.item-layout {
  .icon-image {
    width: 3em;
    height: 3em;
    text-align: center;
    margin-top: .5em;
    margin-right: .5em;
    overflow: hidden;

    img {
      max-height: 100%;
      max-width: 100%;
      height: auto;
      width: auto;
    }

    .icon, img {
      vertical-align: baseline;
    }
  }

  &.minimal-item-layout {
    .icon-image {
      height: 2em;
      width: 2em;
      line-height: 2;
    }
  }

  &.header-item-layout .icon-image {
    margin-top: 0;

    img {
      vertical-align: middle;
    }
  }
}
