// SPDX-FileCopyrightText: 2020 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

.view-mode-switcher {
  display: flex;

  input {
    display: none;
  }

  label {
    color: @control-color;
    line-height: 1;
    background: @low-sat-blue;
    padding: 14/16*.25em 14/16*.5em;
    font-size: 16/12em;
    height: 24/16em; // desired pixel height / font-size
    cursor: pointer;

    &:first-of-type {
      border-top-left-radius: 0.25em;
      border-bottom-left-radius: 0.25em;
    }

    &:last-of-type {
      border-top-right-radius: 0.25em;
      border-bottom-right-radius: 0.25em;
    }

    &:not(:last-of-type) {
      border-right: 1px solid @low-sat-blue-dark;
    }

    i {
      // fix height for Chrome
      display: block;
    }
  }

  input[checked] + label {
    background-color: @control-color;
    color: @text-color-on-icinga-blue;
    cursor: default;
  }
}
