// SPDX-FileCopyrightText: 2020 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

.check-attempt {
  display: flex;
  align-items: center;
  justify-content: center;
}

.check-attempt .ball {
  background-color: @gray-light;

  &:not(:last-child) {
    margin-right: 1/6em;
  }

  &.ball-size-s {
    background-color: @gray-semilight;
  }
}
