<?php

// SPDX-FileCopyrightText: 2019 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

namespace Icinga\Module\Icingadb\Common;

/**
 * @deprecated Use {@see \ipl\Stdlib\BaseFilter} instead. This will be removed with version 1.1
 */
trait BaseFilter
{
    use \ipl\Stdlib\BaseFilter;
}
