<?php

// SPDX-FileCopyrightText: 2021 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

namespace Icinga\Module\Icingadb\Command\Renderer;

/**
 * Interface for Icinga command renderer
 */
interface IcingaCommandRendererInterface
{
}
