<?php

// SPDX-FileCopyrightText: 2022 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

namespace Icinga\Module\Icingadb\Widget;

use ipl\Html\BaseHtmlElement;
use ipl\Html\HtmlElement;
use ipl\Web\Widget\Icon;

class Notice extends BaseHtmlElement
{
    /** @var mixed */
    protected $content;

    protected $tag = 'p';

    protected $defaultAttributes = ['class' => 'notice'];

    /**
     * Create a html notice
     *
     * @param mixed $content
     */
    public function __construct($content)
    {
        $this->content = $content;
    }

    protected function assemble(): void
    {
        $this->addHtml(new Icon('triangle-exclamation'));
        $this->addHtml((new HtmlElement('span'))->add($this->content));
        $this->addHtml(new Icon('triangle-exclamation'));
    }
}
